// RIV
//	0-MoneyPay, 1-TimePay,
// IV
//	0-MoneyDesire, 1-TimeDesire   
// B
//	0-Kill, 1-Up, 2-Down, 3-Same, 4-Threat, 5-Leave, 6-Deal
// RM
//	0-DiffBoth, 1-DiffSelf, 2-WayMarket, 3-Deadline, 4-Kill
// S
//  0-Highest, 1-Lowest, 3-Optimum, 2-Deadline

#define CONS_1 10000
#define PI 3.1415926535

#define MAX_B 10
#define MAX_IV 10
#define MAX_RIV 10
#define MAX_S 10
#define MAX_RM 10
#define MAX_RMBACKT 10

#define RIV_MoneyPay 0
#define RIV_TimePay 1
#define IV_MoneyDesire 0
#define IV_TimeDesire 1
#define B_NOSIGNAL -1
#define B_Kill 0
#define B_Up 1
#define B_Down 2
#define B_Same 3
#define B_Threat 4
#define B_Leave 5
#define B_Deal 6
#define RM_DiffBoth 0
#define RM_DiffSelf 1
#define RM_WayMarket 2
#define RM_Deadline 3
#define RM_Kill 4
#define S_Highest 0
#define S_Lowest 1
#define S_Optimum 2
#define S_Deadline 3

struct type_RIV {
	double val;
	double inc;
};
struct type_IV {	// Internal Variable
	double val;
	double damp;
	double growth;
	double gain[MAX_B];
};
struct type_LI {	// Level of Interest
	double val;
	double damp;
	double growth;
	double bRate;
};
struct type_RM {	// Releasing Mechanism
	double val[MAX_RMBACKT];
	double findMax;
	double findMin;
	int backT;
};
struct type_B {	// behavior value
	double val;
	double oldval;
	double inhibit[MAX_B]; // inhibitory gain
};
struct type_S {
	double val;
};

class CEnvironment;

class CModel
{
public:
	CEnvironment* env;
	int AgentIndex;
	int signal;

	type_RIV m_RIV[MAX_RIV];
	type_IV m_IV[MAX_IV];
	type_LI m_LI[MAX_B];
	type_RM m_RM[MAX_RM];
	type_B m_B[MAX_B];
	type_S m_S[MAX_S];

	int m_NumOfRIV, m_NumOfIV, m_NumOfRM, m_NumOfB, m_NumOfS;
	int m_BOut;

	CModel();
	CModel(CEnvironment* env, int AgentIndex);
	~CModel();
	void InitBehavior(const char buf[20]);
	void UpdateRIV();
	void UpdateIV();
	void UpdateLI();
	void UpdateRM();
	double RMFind(int, double, double);
	double RMTemporalFilterSum(double oldfind[MAX_RMBACKT], int);
	void UpdateB();
	void Modify();
	void UpdateAgent();
};

class CEnvironment
{
public:
	CModel* Agent[2];
	int signal;

	CEnvironment();
	~CEnvironment();
	void InitEnvironment(char argA[20], int, char argB[20], int);
	void UpdateAll();
	void CloseEnvironment();
};

class CCA
{
public:

	//input X
	int X[6][100];

	//state S
	int S[100];

	//output Y
	int Y[6][100];

	//ruletalbe R
//	int R[4096][6][100];

    //data file name
	char* fn;

	//constructor 1
	CCA(char* buf);
    
	//constructor 2
	CCA();
	


	//destructor
	~CCA();

	//initial ruletable
	void InitialCA();

	//generate random number from 0 to upbound-1
	int RandRange(int upbound);

	//generate X[row][] and # bit 1 is equal bc
	void GeneX(int bc,int row,int clean);

	//initial input X from buf(filename)
	void InputX(char* buf);

	//randomly initial input X 
	void InputX();

    //check bit 1 in X neighborhood pos
	int CheckX(int row,int pos);

	//calculate this 4's representation from X[][pos]
	int XtoSRep(int pos);

    //generate S using simple rule
	void GeneS();

	//generate Y by lookup ruletable
	void GeneY();

	//update ruletable according to publish or reward (pf)
	//and update prob is p
	void UpdateRule(int pf,float p);

	int ColShuffle(int col,int bitn,int bitm); 

	//do shuffle to ruletable
	void Shuffle();

	//void Update X
	void UpdateX();



};